from collections import defaultdict
import operator


def count_by_attr(iterable, by):
    d = defaultdict(int)
    for obj in iterable:
        key = getattr(obj, by)
        d[key] += 1
    return d


def dict_by_attr(iterable, by, get=None, default_factory=None):
    if default_factory is not None:
        d = defaultdict(default_factory)
    else:
        d = {}

    for obj in iterable:
        key = getattr(obj, by)
        if get is not None:
            obj = getattr(obj, get)
        d[key] = obj

    return d


def group_by_attr(iterable, by, get=None):
    d = defaultdict(list)
    for obj in iterable:
        key = getattr(obj, by)
        if get is not None:
            obj = getattr(obj, get)
        d[key].append(obj)
    return d


def iter_compare_dicts(dict1, dict2, only_common_keys=False, comparison_op=operator.ne):
    """
    A generator for comparation of values in the given two dicts.

    Yields the tuples (key, pair of values positively compared).

    By default, the *difference* of values is evaluated using the usual != op, but can be changed
    by passing other comparison_op (a function of two arguments returning True/False).

    For example: operator.eq for equal values, operator.is_not for not identical objects.

    You can also require comparison only over keys existing in both dicts (only_common_keys=True).
    Otherwise, you will get the pair with the Python built-in Ellipsis placed for dict with
    that key missing. (Be sure to test for Ellipsis using the 'is' operator.)

    >>> d1 = dict(a=1, b=2, c=3)
    >>> d2 = dict(a=1, b=20, d=4)
    >>> dict(iter_compare_dicts(d1, d2, only_common_keys=True))
    {'b': (2, 20)}
    >>> dict(iter_compare_dicts(d1, d2, only_common_keys=True, comparison_op=operator.eq))
    {'a': (1, 1)}
    >>> dict(iter_compare_dicts(d1, d2))
    {'c': (3, Ellipsis), 'b': (2, 20), 'd': (Ellipsis, 4)}
    >>> dict(iter_compare_dicts(d1, d2, comparison_op=operator.eq))
    {'a': (1, 1), 'c': (3, Ellipsis), 'd': (Ellipsis, 4)}
    """
    keyset1, keyset2 = set(dict1), set(dict2)

    for key in (keyset1 & keyset2):
        pair = (dict1[key], dict2[key])
        if reduce(comparison_op, pair):
            yield key, pair

    if not only_common_keys:
        for key in (keyset1 - keyset2):
            yield key, (dict1[key], Ellipsis)
        for key in (keyset2 - keyset1):
            yield key, (Ellipsis, dict2[key])



import datetime
import os
import bz2

import dill
from django.core.urlresolvers import RegexURLResolver
from django.http.request import HttpRequest
from django.utils.encoding import force_text, force_bytes


#
#                 dump_path = getattr(settings, 'PAGE_DUMP_TEMPLATE_CONTEXT_DIR', None)
#                 if dump_path:
#                     from useful.django.template_context_dumper import dump_template_context
#
#                     dump_template_context(dump_path, template_name, data, context_instance,
#                                           getattr(request, 'urlconf', settings.ROOT_URLCONF))


FIXED_ALLOWED_META = set("""
CONTENT_LENGTH
CONTENT_TYPE
CSRF_COOKIE
DJANGO_SETTINGS_MODULE
GATEWAY_INTERFACE
PATH_INFO
QUERY_STRING
REMOTE_ADDR
REMOTE_HOST
REQUEST_METHOD
SCRIPT_NAME
SERVER_NAME
SERVER_PORT
SERVER_PROTOCOL
SERVER_SOFTWARE
TZ
""".strip().split('\n'))


def dump_template_context(dump_path, template_name, data, context_instance, urlconf):

    dicts_copy = [d.copy() for d in context_instance.dicts]

    for ctx in context_instance:

        if 'messages' in ctx:
            ctx['messages'] = list(ctx['messages'])

        if 'csrf_token' in ctx:
            ctx['csrf_token'] = force_text(ctx['csrf_token'])

        if 'request' in ctx:
            req = ctx['request']
            new_req = HttpRequest()
            for attr in 'GET', 'POST', 'COOKIES', 'path', 'path_info', 'method':
                setattr(new_req, attr, getattr(req, attr))

            new_req.META = dict((k, force_text(v)) for k, v in req.META.items()
                                if (k in FIXED_ALLOWED_META or k.startswith('HTTP_')))

            ctx['request'] = new_req

        if 'sql_queries' in ctx:
            del ctx['sql_queries']

    resolver = RegexURLResolver(r'^/', urlconf)
    # enforce population
    resolver.reverse_dict
    resolver._namespace_dict = {}
    if hasattr(resolver, '_urlconf_module'):
        delattr(resolver, '_urlconf_module')

    for rdict in resolver._reverse_dict.values():
        for key in rdict.keys():
            if callable(key):
                del rdict[key]

    now = datetime.datetime.now()

    out_filename = now.strftime('%Y%m%d-%H%M%S-')
    out_filename += force_bytes(template_name).lower().replace('/', '-SLASH-').replace('.', '-DOT-')
    out_filename += '.template-context-dill.bz2'

    if not os.path.exists(dump_path):
        os.makedirs(dump_path, mode=0755)

    out_filepath = os.path.join(dump_path, out_filename)

    out_file = bz2.BZ2File(out_filepath, 'w')

    try:
        dump = dict(
            now = now,
            template_name = template_name,
            data = data,
            context_instance = context_instance,
            resolver = resolver,
        )

        dill.dump(dump, out_file)
        out_file.close()

    except:
        out_file.close()
        os.unlink(out_filepath)
        raise

    context_instance.dicts = dicts_copy
